/*
* ============================================================================
*  Name        : mccexengine.cpp
*  Part of     : mccexample / mccexengine
*  Description : MCC example engine implementation
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.1.1
*/

#include <mmccinterface.h>
#include <mmcccodecinformation.h>

#include "mccexengine.h"
#include "mcceventhandler.h"
#include "maudiocontrol.h"
#include "loopbackaudiocontrol.h"
#include "loopbackvideocontrol.h"
#include "mccexenginecamerainfo.h"
#include "mccexpanic.pan"

// LOCAL CONSTANTS
const TInt KExDefaultMediaQos = 184;
const TInt KExDefaultRtpPort = 5000;

// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CMccExEngine::CMccExEngine
// ---------------------------------------------------------------------------
//
CMccExEngine::CMccExEngine()
    {
    iNwSettings.iMediaQoS = KExDefaultMediaQos;
    iNwSettings.iRtpPort = KExDefaultRtpPort;
    iNwSettings.iRtcpEnabled = ETrue;
    
    TInetAddr exampleAddr( INET_ADDR( 127,0,0,1 ), KExDefaultRtpPort );
    iNwSettings.iRemoteAddress = exampleAddr;
    }


// ---------------------------------------------------------------------------
// CMccExEngine::~CMccExEngine
// ---------------------------------------------------------------------------
//
EXPORT_C CMccExEngine::~CMccExEngine()
    {
    delete iInterface;
    delete iEventHandler;
    iCodecSettings.Close();
    }


// ---------------------------------------------------------------------------
// CMccExEngine::NewL
// ---------------------------------------------------------------------------
//
EXPORT_C CMccExEngine* CMccExEngine::NewL()
    {
    CMccExEngine* self = CMccExEngine::NewLC();
    CleanupStack::Pop( self );
    return self;
    }


// ---------------------------------------------------------------------------
// CMccExEngine::NewLC
// ---------------------------------------------------------------------------
//
EXPORT_C CMccExEngine* CMccExEngine::NewLC()
    {
    CMccExEngine* self = new ( ELeave ) CMccExEngine();
    CleanupStack::PushL( self );
    self->ConstructL();
    return self;
    }


// ---------------------------------------------------------------------------
// CMccExEngine::CreateLoopbackAudioControlL
// ---------------------------------------------------------------------------
//
EXPORT_C MAudioControl* CMccExEngine::CreateLoopbackAudioControlL(
    MMccExEngineObserver& aObserver )
    {
    // Assert the proper state
    __ASSERT_ALWAYS( iInterface && iEventHandler && iCodecSettings.Count(),
                     MccExPanic( EMccExNullPtr ) );
    
    CLoopbackAudioControl* ctrl = 
        CLoopbackAudioControl::NewL( *iInterface, *iEventHandler,
            aObserver, iNwSettings, iCodecSettings[0] );
    
    return ctrl;
    }


// ---------------------------------------------------------------------------
// CMccExEngine::CreateLoopbackVideoControlL
// ---------------------------------------------------------------------------
//
EXPORT_C MVideoControl* CMccExEngine::CreateLoopbackVideoControlL(
    MMccExEngineObserver& aObserver,
    const TRect& aRect,
    const TSize& aResolution,
    const TVideoDeviceParameters& aDevParams )
    {
    // Assert the proper state
    __ASSERT_ALWAYS( iInterface && iEventHandler,
            MccExPanic( EMccExNullPtr ) );
    
    CLoopbackVideoControl* ctrl = 
        CLoopbackVideoControl::NewL( *iInterface, *iEventHandler,
                aObserver, aRect, aResolution, aDevParams, iNwSettings );
    
    return ctrl;
    }


// ---------------------------------------------------------------------------
// CMccExEngine::ResetEngineL
// ---------------------------------------------------------------------------
//
EXPORT_C void CMccExEngine::ResetEngineL()
    {
    TInetAddr exampleAddr( INET_ADDR( 127,0,0,1 ), KExDefaultRtpPort );
    
    iNwSettings = TNetworkSettings(
        KExDefaultMediaQos, KExDefaultRtpPort, ETrue, exampleAddr );
    iCodecSettings.Reset();
    
    delete iInterface;
    delete iEventHandler;
    iInterface = NULL;
    iEventHandler = NULL;
    ConstructL();
    }


// ---------------------------------------------------------------------------
// CMccExEngine::SupportedCamerasL
// ---------------------------------------------------------------------------
//
EXPORT_C TInt CMccExEngine::SupportedCamerasL()
    {
    // Assert the correct state of the engine.
    __ASSERT_ALWAYS( iInterface && iEventHandler,
            MccExPanic( EMccExNullPtr ) );
    
    TInt count( 0 );
    TPckgBuf<TInt> buffer( count );
    User::LeaveIfError(
        iInterface->GetParameter( 0, 0, 0, 0, KMccCamCount, buffer ) );
    
    return buffer();
    }


// ---------------------------------------------------------------------------
// CMccExEngine::CameraInformationL
// ---------------------------------------------------------------------------
//
EXPORT_C void CMccExEngine::CameraInformationL( TInt aCamera,
    TCameraInfo& aInfo )
    {
    // For getting the camera information from MCC you can use the following
    // code snippet. Unfortunately when interrogating camera information via
    // MCC you cannot specify the camera whose information you want.
    // Thus we use our own little helper get *specific* camera information.
    
    /* Code snippet for getting cam info via MCC.
     * 
     * TPckgBuf<TCameraInfo> buffer( aInfo );
     * User::LeaveIfError(
     *      iInterface->GetParameter( 0, 0, 0, 0, KMccCamInfo, buffer ) );
     */
    
    CMccExEngineCameraInfo* cam( CMccExEngineCameraInfo::NewLC( aCamera ) );
    cam->CameraInformation( aInfo );
    CleanupStack::PopAndDestroy( cam );
    }


// ---------------------------------------------------------------------------
// CMccExEngine::NetworkSettings
// ---------------------------------------------------------------------------
//
EXPORT_C void CMccExEngine::NetworkSettings(
    TNetworkSettings& aSettings ) const
    {
    aSettings = iNwSettings;
    }


// ---------------------------------------------------------------------------
// CMccExEngine::SetNetworkSettings
// ---------------------------------------------------------------------------
//
EXPORT_C void CMccExEngine::SetNetworkSettings(
    const TNetworkSettings& aSettings )
    {
    iNwSettings = aSettings;
    }


// ---------------------------------------------------------------------------
// CMccExEngine::AudioCodecSettingsL
// ---------------------------------------------------------------------------
//
EXPORT_C void CMccExEngine::AudioCodecSettingsL(
    RArray<TCodecSettings>& aCodecs ) const
    {
    // Assert that we have at least some settings stored.
    __ASSERT_ALWAYS( iCodecSettings.Count(), User::Leave( KErrNotReady ) );
    
    aCodecs.Reset();
    const TInt count = iCodecSettings.Count();
    for( TInt k = 0; k < count; k++ )
        {
        TCodecSettings setting = iCodecSettings[k];
        aCodecs.AppendL( setting );
        }
    }


// ---------------------------------------------------------------------------
// CMccExEngine::SetAudioCodecSettingsL
// ---------------------------------------------------------------------------
//
EXPORT_C void CMccExEngine::SetAudioCodecSettingsL(
    const RArray<TCodecSettings>& aCodecs )
    {
    // Assert that we have at least some settings to store.
    __ASSERT_ALWAYS( aCodecs.Count(), User::Leave( KErrArgument ) );
    
    iCodecSettings.Reset();
    const TInt count = aCodecs.Count();
    for( TInt k = 0; k < count; k++ )
        {
        TCodecSettings setting = aCodecs[k];
        iCodecSettings.AppendL( setting );
        }
    }

// ---------------------------------------------------------------------------
// CMccExEngine::ConstructL
// ---------------------------------------------------------------------------
//
void CMccExEngine::ConstructL()
    {
    iEventHandler = CMccEventHandler::NewL();
    iInterface = CMccInterface::NewL( *iEventHandler );
    
    // CleanupResetAndDestroyPushL comes from mmfcontrollerpluginresolver.h
    // and that enables us to do a bit more elegant cleanup with pointer
    // arrays.
    RPointerArray<CMccCodecInformation> capas;
    CleanupResetAndDestroyPushL( capas );
    
    User::LeaveIfError( iInterface->GetCapabilities( capas ) );
    FilterCapabilities( capas, KUidMediaTypeAudio );
    
    const TInt codecAmt( capas.Count() );
    TCodecSettings setting;
    CMccCodecInformation* codec( NULL );
    
    for( TInt k = 0; k < codecAmt; k++ )
        {
        // We do not want the "virtual codecs" on this list as they are wholly
        // different beasts.
        codec = capas[k];
        if ( codec->SdpName().Compare( KTelephoneEvent ) && 
             codec->SdpName().Compare( KRedSdpName ) && 
             codec->SdpName().Compare( KCnSdpName ) )
            {
            setting.iCodecName = codec->SdpName();
            setting.iJitterBufferSize = codec->JitterBufBufferLength();
            setting.iMaxPTime = codec->MaxPTime();
            setting.iPTime = codec->PTime();
            setting.iVadCn = codec->VAD();
            
            iCodecSettings.AppendL( setting );
            }
        }
    
    CleanupStack::PopAndDestroy( &capas );
    }


// ---------------------------------------------------------------------------
// CMccExEngine::FilterCapabilities
// ---------------------------------------------------------------------------
//
void CMccExEngine::FilterCapabilities(
    RPointerArray<CMccCodecInformation>& aCapabilities,
    const TUid& aMediaType ) const
    {
    TInt k = aCapabilities.Count();
    while( k )
        {
        k -= 1;
        if ( aCapabilities[k]->Type() != aMediaType )
            {
            CMccCodecInformation* codec = aCapabilities[k];
            aCapabilities.Remove( k );
            delete codec;
            }
        }
    }
